/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Mozilla Penelope project.
 *
 * The Initial Developer of the Original Code is
 * QUALCOMM Incorporated.
 * Portions created by QUALCOMM Incorporated are
 * Copyright (C) 2007-2010 QUALCOMM Incorporated. All Rights Reserved.
 *
 * Contributor(s):
 *     Mark Charlebois <mcharleb@qualcomm.com> original author
 *     Jeff Beckley <beckley@qualcomm.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var PenelopeGlobal = { 
    onLoad: function() 
    { 
        // quit if this function has already been called
        if (arguments.callee.done)
            return;

        // flag this function so we don't do the same thing twice
        arguments.callee.done = true;

        dump("in PenelopeGlobal.onLoad\n");
        updateSelectedTextUrlLabels("penelopeDoSelectedTextUrl");

        setKeymap();
    } 
}; 

//load event handler
window.addEventListener("load", function(e) { PenelopeGlobal.onLoad(e); }, false); 

function updateSelectedTextUrlLabels(aIdPrefix)
{
    // Get the preferences object
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefBranch);

    // Get the URL prefs
    for (var urlnum=1; urlnum<=6; urlnum++)
    {
        var menu = document.getElementById(aIdPrefix+urlnum);
        if (menu)
        {
            try
            {
                var url = prefs.getCharPref("penelope.selectedTextURL"+urlnum);
                var name = url.split(",")[0];
                if (name)
                {
                    menu.setAttribute('label', name);
                    menu.setAttribute('hidden', 'false');
                }
                else
                {
                    menu.setAttribute('hidden', 'true');
                }
            }
            catch(e)
            {
                menu.setAttribute('hidden', 'true');
            }
        }
    }
}

function openURL(urlToOpen) 
{
    var uri = Components.classes["@mozilla.org/network/io-service;1"]
              .getService(Components.interfaces.nsIIOService)
              .newURI(urlToOpen, null, null);

    var protocolSvc = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
                      .getService(Components.interfaces.nsIExternalProtocolService);
    protocolSvc.loadUrl(uri);
}

function penelopeGetSelectedText()
{
    var selection = "";

    try 
    {
        var focusedWindow = document.commandDispatcher.focusedWindow;
        selection = focusedWindow.getSelection().toString();
    }
    catch (e)
    {
        dump("No text selected\n");
    }

    return selection;
}


function doSelectedTextUrl(number)
{
    // Get the selected text
    // Get the preferences object
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefBranch);

    var str = "penelope.selectedTextURL"+number;
    try 
    {
        var entry = prefs.getCharPref(str);
        if (entry)
        {
            var selectedText = penelopeGetSelectedText();
            var sep = entry.indexOf(",");
            var url = entry.substring(sep+1, entry.length);
            url = url.replace("%s", selectedText);
            openURL(url);
        }
    }
    catch(e) 
    { 
        alert(str+" is not set or is invalid");
    }
}

function openDirectoryServices()
{
    // uses "toOpenWindowByType" function provided by utilityOverlay.js
    // which is included by most clients.
    toOpenWindowByType("mail:addressbook", "chrome://messenger/content/addressbook/addressbook.xul");
}

function directoryServicesQuery()
{
    // uses "toOpenWindowByType" function provided by utilityOverlay.js
    // which is included by most clients.
    toOpenWindowByType("mail:addressbook", "chrome://messenger/content/addressbook/addressbook.xul");
}


function penelopeGetPlatform() 
{
    var tmp = navigator.platform.toLowerCase();
    var platform = "windows";

    if (tmp.indexOf("linux") != -1) 
    {
        platform = "linux";
    }
    else if (tmp.indexOf("mac") != -1) 
    {
        platform = "mac";
    }

    return platform;
}

function setKeymap()
{
    dump("Setting keymap...\n");
    var prefs = Components.classes["@mozilla.org/preferences-service;1"].
                getService(Components.interfaces.nsIPrefBranch);

    // If the user wants the penelope keymap, get the name
    var usePenelopeKeyset = prefs.getBoolPref("penelope.useKeymap");
    var selectedKeyset = "thunderbird";
    if (usePenelopeKeyset)
    {
        selectedKeyset = prefs.getCharPref("penelope.keymap").toLowerCase();
    }

    var strbundle;

    if (selectedKeyset == "thunderbird")
    {
        strbundle = document.getElementById("penelopeThunderbirdKeys");
    }
    else
    {
        var platform = penelopeGetPlatform();
        dump("Loading keyset for "+platform+"\n");

        switch(platform)
        {
        case "mac":
            strbundle = document.getElementById("penelopeMacKeys");
            break;
        case "windows":
        case "linux":
        default:
            strbundle = document.getElementById("penelopeWindowsKeys");
            break;
        }
    }

    // The particular window may not have the global overlay applied
    // so the strbundles may not be defined
    if (strbundle)
    {
        loadKeySet(strbundle.strings);
    }
}

function loadKeySet(strings)
{
    while (strings.hasMoreElements())
    {
        var node = strings.getNext().QueryInterface(Components.interfaces.nsIPropertyElement);;
        var key = document.getElementById(node.key);
        if (key)
        {
            setKey(key, node.value);
        }
    }
}

function setKey(key, value)
{
    var properties = value.split("][");

    if(properties[0] == "!")
    {
        key.setAttribute("disabled", true);
        key.setAttribute("key", "");
        key.setAttribute("modifiers", "");
    }
    else 
    {
        if (properties[0])
        {
            key.setAttribute("modifiers", properties[0]);
        }

        if(properties[1]) 
        {
            key.setAttribute("key", properties[1]);
            try
            {
                key.attributes.removeNamedItem("keycode");
            }
            catch (e) {}
        }
        else if(properties[2]) 
        {
            key.setAttribute("keycode", properties[2]);
            try
            {
                key.attributes.removeNamedItem("key");
            }
            catch (e) {}
        }
        else
        {
            alert("Error: No key specified for "+key.id+". Disabling entry.");
            key.setAttribute("disabled", true);
        }
    }
}

function GetSelectedFolderURIForWin(win)
{
    try
    {
        dump("WINDOW: "+win.document.title+" ");
        var uri = win.GetSelectedMsgFolders()[0].URI
        dump("WIN URI = "+uri+"\n");
        return uri;
    }
    catch(e)
    {
        dump("Caught exception in GetSelectedFolderURIForWin\n");
    }

    // If the selected folder can't be found it may be due to the window not being fully
    // open yet.  In that case the folder to open will be the first window argument.
    if (win.arguments && win.arguments.length > 0)
        return win.arguments[0];

    return null;
}

function goMailbox(folderUri, userAction, forceNewWindow, forceNewTab, leaveInBackground, msgHdr)
{
    var openInTab = !("gIsMailboxesWindow" in window && window.gIsMailboxesWindow) &&
                    (forceNewTab || (!forceNewWindow && gPrefBranch.getBoolPref("penelope.mailbox.openInTab")));

    if (openInTab)
    {
        var tabmail = document.getElementById('tabmail');
        
        var numTabs = tabmail.tabInfo.length;
        var tabIndex;
        for (tabIndex = 0; tabIndex < numTabs; tabIndex++)
        {
            try
            {
                var tabInfo = tabmail.tabInfo[tabIndex];
                var tabFolderDisplay = tabInfo.folderDisplay;
                var tabFolder = tabFolderDisplay.displayedFolder;
                var tabFolderUri = tabFolder.URI;

                if (tabInfo.mode.name == "folder" && folderUri == tabFolderUri)
                {
                    if (userAction && tabIndex == tabmail.tabContainer.selectedIndex)
                        continue;

                    if (!leaveInBackground)
                    {
                        // Select the already existing matching tab
                        tabmail.switchToTab(tabIndex);
                        if (msgHdr)
                            tabFolderDisplay.selectMessage(msgHdr);
                        window.focus();
                        function setThreadTreeFocus()
                        {
                            GetThreadTree().focus();
                        }
                        setTimeout(setThreadTreeFocus, 1);
                    }
                    // Since this is a background tab it will set the
                    // needsFUMLUBSelection flag so that the next time the
                    // tab comes to the foregound it will do FUMLUB selection
                    if (tabIndex != tabmail.tabContainer.selectedIndex)
                        SelectFUMLUB(tabFolder);
                    return;
                }
            }
            catch (e) {}
        }

        // Didn't find a match, so open it up in a new tab
        var msgFolder = GetMsgFolderFromUri(folderUri, false);
        var args = {folder: msgFolder, background: leaveInBackground};
        if (msgHdr)
        {
            args.msgHdr = msgHdr;
            args.forceSelectMessage = true;
            window.ignoreNextFUMLUB = true;
            msgFolder.lastMessageLoaded = msgHdr.messageKey;
        }
        gOpeningMailboxTab = true;
        tabmail.openTab("folder", args);
        gOpeningMailboxTab = false;
        if (!leaveInBackground)
            window.focus();
    }
    else
    {
        // Grab window list; for all windows, pass null as the parameter.
        var mediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
        var windows = mediator.getZOrderDOMWindowEnumerator("mail:3pane", true);

        while (windows.hasMoreElements()) 
        {
            var win = windows.getNext();

            // If this is a user action (e.g. double-click or Enter/Return), then don't consider the current
            // window a match because it will always match (because it is selected on the right)
            if (userAction && win == window)
                continue;

            if (win.gIsMailboxesWindow == true)
                continue;

            var tabmail = win.document.getElementById('tabmail');
            if (tabmail)
            {
                var tabInfo = tabmail.tabInfo[0];
                var tabFolderDisplay = tabInfo.folderDisplay;
                var tabFolder = tabFolderDisplay.displayedFolder;
                var tabFolderUri = tabFolder.URI;

                if (tabInfo.mode.name == "folder" && folderUri == tabFolderUri)
                {
                    tabmail.switchToTab(0);
                    if (msgHdr)
                        win.gFolderDisplay.selectMessage(msgHdr);
                    if (!leaveInBackground)
                    {
                        win.GetThreadTree().focus();
                        win.focus();
                    }
                    return;
                }
            }
        }

        var newWin = window.openDialog("chrome://messenger/content/", "_blank",
                                       "chrome,all,dialog=no", folderUri);
        if (msgHdr)
        {
            newWin.ignoreNextFUMLUB = true;
            
            // Need to set lastMessageLoaded to select the message so that we
            // don't get multiple calls to SelectFUMLUB()
            var folder = GetMsgFolderFromUri(folderUri, false);
            if (folder)
                folder.lastMessageLoaded = msgHdr.messageKey;
        }
    }
}

function OpenSpecialMailbox(flag)
{
    var foundFolder = null;
    
    if (gDBView)
        foundFolder = gDBView.msgFolder.server.rootMsgFolder.getFolderWithFlags(flag);
    
    if (!foundFolder)
    {
        foundFolder = accountManager.localFoldersServer.rootMsgFolder.getFolderWithFlags(flag);
        if (!foundFolder)
            return false;
    }

    goMailbox(foundFolder.URI, false, false);

    return true;
}

function openInbox() 
{
    OpenSpecialMailbox(Components.interfaces.nsMsgFolderFlags.Inbox);
}

function openOutbox() 
{
    OpenSpecialMailbox(Components.interfaces.nsMsgFolderFlags.SentMail);
}

function openJunk() 
{
    OpenSpecialMailbox(Components.interfaces.nsMsgFolderFlags.Junk);
}

function openTrash() 
{
    OpenSpecialMailbox(Components.interfaces.nsMsgFolderFlags.Trash);
}

function addEmail()
{
    try 
    {
        if (gDBView.URIForFirstSelectedMessage) 
        {
            var hdr = messenger.msgHdrFromURI(gDBView.URIForFirstSelectedMessage);
            var hdrParser = Components.classes["@mozilla.org/messenger/headerparser;1"].getService(Components.interfaces.nsIMsgHeaderParser);
            var addresses = {};
            var names = {};
            var fullNames = {};
            var numAddresses = hdrParser.parseHeadersWithArray(hdr.author, addresses, names, fullNames);

            if (numAddresses <= 0)
            {
                alert("Add email to Addressbook failed");
            }

            var emailAddress = addresses.value[0];
            var displayName = names.value[0];
            window.openDialog("chrome://messenger/content/addressbook/abNewCardDialog.xul",
                              "",
                               "chrome,resizable=no,titlebar,modal,centerscreen",
                              {primaryEmail: emailAddress, displayName:displayName});
        }
    }
    catch(e) { alert("No message selected"); }
}

function penelopeUpdatePriorityMenu(aPopup)
{
    // Bug in Mac radio menu items doesn't ensure only one item is selected,
    // so we have to do that ourselves
    const MP = Components.interfaces.nsMsgPriority;
    for (var p = MP.lowest; p <= MP.highest; p++)
        aPopup.getElementsByAttribute("value", p)[0].setAttribute("checked", "false");

    var msgHdr = gDBView.hdrForFirstSelectedMessage;
    
    if (msgHdr) 
    {
        var priority = msgHdr.priority;
        
        if (priority < MP.lowest || priority > MP.highest)
            priority = MP.normal;
        aPopup.getElementsByAttribute("value", priority)[0].setAttribute("checked", "true");
    }
}

function penelopePriorityMenuSelect(target)
{
    var selectedMessages = gFolderDisplay.selectedMessages;
    var priority = target.getAttribute('value');
    
    for (var i = 0; i < selectedMessages.length; ++i)
        selectedMessages[i].priority = priority;
}

function penelopeInitStatusTags(aPopup)
{
    var currentValue = "Read";  
    var msgHdr = gDBView.hdrForFirstSelectedMessage;

    if (msgHdr.flags & Components.interfaces.nsMsgMessageFlags.Replied)
        currentValue = "Replied";
    else if (msgHdr.flags & Components.interfaces.nsMsgMessageFlags.Forwarded)
        currentValue = "Forwarded";
    else if (msgHdr.isRead == false)
        currentValue = "Unread";
    aPopup.getElementsByAttribute("value", currentValue)[0].setAttribute("checked", "true");
}

function penelopeStatusMenuSelect(target)
{
    var markRead = true;
    var markReplied = false;
    var markForwarded = false;

    switch (target.getAttribute('value'))
    {
    case "Unread":
        markRead = false;
        break;
    case "Read":
        break;
    case "Replied":
        markReplied = true;
        break;
    case "Forwarded":
        markForwarded = true;
        break;
    case "Redirected":
    case "Sent":
    case "Unsent":
        alert("Not implemented");
        return;
    default:
        alert("Unknown status");
        return;
    }

    var selectedMessages = gFolderDisplay.selectedMessages;
    for (var i = 0; i < selectedMessages.length; ++i)
    {
        var msgKey = selectedMessages[i].messageKey;
        gDBView.db.MarkRead(msgKey, markRead, null);
        gDBView.db.MarkReplied(msgKey, markReplied, null);
        gDBView.db.MarkForwarded(msgKey, markForwarded, null);
    }
}

function penelopeMailboxMenuFilter(aFolder)
{
    var server = aFolder.server;

    // Don't show POP3 accounts that go to the Global Inbox
    if (server.type == "pop3" && !server.canFileMessagesOnServer)
        return false;
        
    return true;
}

function onShowingMailboxMenu(aMailboxPopup)
{
    var filters = aMailboxPopup._filters;
    
    if (filters && !filters["penelopeMailboxMenu"])
        filters["penelopeMailboxMenu"] = penelopeMailboxMenuFilter;
}

function penelopeInitMailContextMenu(aEvent, aPopupMenu)
{
    // Let the default TB code go first as we'll muck with items once its done
    fillMailContextMenu(aEvent);

    // If the popupshowing was for a submenu, we don't need to do anything.
    if (aEvent.target != aEvent.currentTarget)
        return true;

    // Show Selected Text URL menu only in the message pane,
    // and enable only if there is something selected
    var inMessagePane = !gContextMenu.inThreadPane;
    var isContentSelected = gContextMenu.isContentSelected;
    ShowMenuItem("selectedTextUrlMenu", inMessagePane);
    EnableMenuItem("selectedTextUrlMenu", inMessagePane && isContentSelected);
    ShowMenuItem("mailContext-sep-open", inMessagePane);

    // Redirect only when doing mail items
    var showMailItems = !gContextMenu.hideMailItems;
    ShowMenuItem("mailContext-redirect", showMailItems);
    ShowMenuItem("mailContext-sep-reply", showMailItems);

    // Hide some items we never want
    ShowMenuItem("mailContext-archive", false);
    ShowMenuItem("mailContext-replyList", false);
    ShowMenuItem("mailContext-saveAs", false);
    ShowMenuItem("mailContext-printpreview", false);
    
    return true;
}

function penelopeRedirect()
{
    composeMsgByType(Components.interfaces.nsIMsgCompType.Redirect);
}

function updateMailboxCss()
{
    // Add new CSS rules for column padding, based on hidden user prefs.

    var styleSheet = document.styleSheets[document.styleSheets.length - 1];
    var ruleList = styleSheet.cssRules;

    const ruleStart    = 'treechildren::-moz-tree-cell-text(';
    const ruleMidLeft  = ') {padding-left: ';
    const ruleMidRight = ') {padding-right: ';
    var   rulePad      = 20;	// default rule size. Will be replaced by setting later.
    const ruleEnd      = 'px; }';

    var newRuleString; // A final rule will be ruleStart + thisColumn + ruleMid + rulePad + ruleEnd

    // First the left padding
    var normalColPadding = gPrefBranch.getIntPref("penelope.mailbox.columnPadLeft");

    // We'll save the array of columns we want to modify in a string pref (a comma-separated list)
    var columnsToPad = gPrefBranch.getCharPref("penelope.mailbox.columnsToPadLeft");
    // And then split it into an array:
    var columnsArray = columnsToPad.split(",");

    for (var colIndex = 0; colIndex < columnsArray.length; colIndex++)
    {
        // Now add the new rule for this column at the end of the list of rules
        var thisColumn = columnsArray[colIndex]; // thisColumn is a string, e.g. "whoCol"

        rulePad = normalColPadding;
        try
        {
            rulePad = gPrefBranch.getIntPref("penelope.mailbox.columnPadLeft." + thisColumn);
        }
        catch(e){}
        try
        {
            newRuleString = ruleStart + thisColumn + ruleMidLeft + rulePad + ruleEnd;
            styleSheet.insertRule(newRuleString, ruleList.length);
        }
        catch(e)
        {
            dump("Error setting css rule for column" +e+ "\n");
        }
    }

    // Now the right padding, but only if we're showing mailbox lines
    if (gPrefBranch.getBoolPref("penelope.mailbox.showLines"))
    {
        normalColPadding = gPrefBranch.getIntPref("penelope.mailbox.columnPadRight");

        // We'll save the array of columns we want to modify in a string pref (a comma-separated list)
        columnsToPad = gPrefBranch.getCharPref("penelope.mailbox.columnsToPadRight");
        // And then split it into an array:
        columnsArray = columnsToPad.split(",");

        for (colIndex = 0; colIndex < columnsArray.length; colIndex++)
        {
            // Now add the new rule for this column at the end of the list of rules
            thisColumn = columnsArray[colIndex]; // thisColumn is a string, e.g. "whoCol"

            rulePad = normalColPadding;
            try
            {
                rulePad = gPrefBranch.getIntPref("penelope.mailbox.columnPadRight." + thisColumn);
            }
            catch(e){}
            try
            {
                newRuleString = ruleStart + thisColumn + ruleMidRight + rulePad + ruleEnd;
                styleSheet.insertRule(newRuleString, ruleList.length);
            }
            catch(e)
            {
                dump("Error setting css rule for column" +e+ "\n");
            }
        }
    }
}

function enableMailboxLines()
{
    // Check to see if the setting for showing Mailbox lines is set (default is ON)
    // and modify the CSS appropriately

    if (gPrefBranch.getBoolPref("penelope.mailbox.showLines"))
    {
        var styleSheet = document.styleSheets[document.styleSheets.length - 1];
        var ruleList = styleSheet.cssRules;

        var color = gPrefBranch.getCharPref("penelope.mailbox.showLines.color") || "#C0C0C0";// default value is #C0C0C0

        var rules = []; // rules is an array of rules.

        const r0Start = '#threadTree > treechildren::-moz-tree-cell {border-right: 1px solid ';
        const r0End   = ';}';
        rules[0] = r0Start + color + r0End;

        const r1Start = '#threadTree treechildren::-moz-tree-column() {border-right: 1px solid ';
        const r1End   = ';}';
        rules[1] = r1Start + color + r1End;

        const r2Start = '#threadTree treechildren::-moz-tree-row() {border-bottom: 1px solid !important; border-bottom-color:';
        const r2End   = ' !important;}';
        rules[2] = r2Start + color + r2End;

        rules[3] = '#threadTree > treechildren::-moz-tree-cell(statusCol) {border-right: none !important;}';
        rules[4] = '#threadTree > treechildren::-moz-tree-cell(priorityCol) {border-right: none !important;}';
        rules[5] = '#threadTree treechildren::-moz-tree-column(statusCol) {border-right: none !important;}';
        rules[6] = '#threadTree treechildren::-moz-tree-column(priorityCol) {border-right: none !important;}';

        // Add the rules
        var index = 0;
        for (index = 0; index <= rules.length; index++)
        {
            try
            {
                styleSheet.insertRule(rules[index], ruleList.length);
            }
            catch(e)
            {
                dump("Error adding css rule for line index#" +index +e+ "\n");
            }   
        }
    }
}

function updateIconColumns()
{
    var fix = true;
    var width = 24;
    try {
        fix = gPrefBranch.getBoolPref("penelope.mailbox.fixIconColumns");
        width = gPrefBranch.getIntPref("penelope.mailbox.fixIconColumns.width");
    } catch (e) {}
    
    try {
        var iconColumns = ["threadCol", "flaggedCol", "statusCol", "unreadButtonColHeader", "priorityCol", "attachmentCol", "junkStatusCol"];
        
        for (var i = 0; i < iconColumns.length; i++)
        {
            var col = document.getElementById(iconColumns[i]);
            col.setAttribute("fixed", fix? "true" : "false");
            if (fix)
                col.setAttribute("width", width.toString());
        }
    } catch (e) {}
}
