/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Mozilla Penelope project.
 *
 * The Initial Developer of the Original Code is
 * QUALCOMM Incorporated.
 * Portions created by QUALCOMM Incorporated are
 * Copyright (C) 2007-2010 QUALCOMM Incorporated. All Rights Reserved.
 *
 * Contributor(s):
 *     Mark Charlebois <mcharleb@qualcomm.com> original author
 *     Jeff Beckley <beckley@qualcomm.com>
 *     Matt Dudziak <mdudziak@qualcomm.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var PenelopeMessenger = { 
    onLoad: function() 
    { 
        // TODO If the window has already been created, close the new one and focus the old one

        // quit if this function has already been called
        if (arguments.callee.done)
            return;

        // flag this function so we don't do the same thing twice
        arguments.callee.done = true;

        // update the mailbox and transfer menus
        setTimeout(delayedOnLoadPenelopeMessenger, 50); 

        // We only need to set the sound once
        setNewMailSound();

        // Set up an observer for when the view has been created
        // in order to register the Group Select column
        var ObserverService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);
        ObserverService.addObserver(CreateDbObserver, "MsgCreateDBView", false);
    }
}; 

//load event handler
window.addEventListener("DOMContentLoaded", function(e) { PenelopeMessenger.onLoad(e); }, false); 


function saveOpenWindows(forceSave)
{
    // We don't save windows while the restore is currently going on.
    // That way we don't write the entries while we're trying to read them up.
    if (!forceSave && gRestoreInProgress)
        return;
        
    var sessionInfo = { windows: [{}] };
        
    var mediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                             .getService(Components.interfaces.nsIWindowMediator);

    // Grab window list from back to front so that when we restore the last window
    // restored will show up in front
    var windows = mediator.getZOrderXULWindowEnumerator(null, false);

    var openWindowNum;
    for (openWindowNum = 0; windows.hasMoreElements(); )
    {
        var win = windows.getNext();

        const Ci = Components.interfaces;
        if(!(win instanceof Ci.nsIXULWindow)) 
        {
            dump("Found invalid item\n");
            continue;
        }

        // Don't save off modal windows
        if (win.chromeFlags & Ci.nsIWebBrowserChrome.CHROME_MODAL)
            continue;

        var requestor = win.docShell.QueryInterface(Ci.nsIInterfaceRequestor);
        var mywin = requestor.getInterface(Ci.nsIDOMWindow);
        if (mywin)
        {
            var siWin;
            var saveIt = true;
            var folderUri = null;
            var messageUri = null;
            var messageKey = nsMsgKey_None;
            var docEl = mywin.document.documentElement;
            var wintype = docEl.getAttribute('windowtype');
            if (!wintype)
                wintype = docEl.getAttribute('id');
            switch (wintype)
            {
            case "mail:3pane":
                var tabmail = mywin.document.getElementById('tabmail');
                var numTabs = tabmail.tabInfo.length;
                if (openWindowNum >= sessionInfo.windows.length)
                    sessionInfo.windows.push(new Object());
                siWin = sessionInfo.windows[openWindowNum];
                siWin.tabs = [];
                siWin.tabSelected = tabmail.tabContainer.selectedIndex;
                var tabIndex;
                for (tabIndex = 0; tabIndex < numTabs; tabIndex++)
                {
                    var tabInfo = tabmail.tabInfo[tabIndex];
                    var tabType = tabInfo.mode.name;
                    if (tabType != "folder" && tabType != "message")
                    {
                        if (tabIndex < siWin.tabSelected)
                            siWin.tabSelected--;
                    }
                    else
                    {
                        var tabFolderUri = folderUri;
                        try
                        {
                            tabFolderUri = tabInfo.folderDisplay.displayedFolder.URI;
                        }catch(e){}
                        if (!tabIndex)
                            folderUri = tabFolderUri;
                        else
                        {
                            siWin.tabs.push(new Object());
                            var siTab = siWin.tabs[siWin.tabs.length - 1];
                            siTab.type = tabType;
                            siTab.folderUri = tabFolderUri;
                            if (tabType == "message")
                                siTab.messageKey = tabInfo.folderDisplay.selectedMessage.messageKey;
                        }
                    }
                }
                break;
            case "mail:messageWindow":
                var hdr = mywin.gMessageDisplay.displayedMessage;
                messageKey = hdr.messageKey;
                folderUri = hdr.folder.URI;
                break;
            case "msgcompose":
                messageUri = mywin.gMsgCompose.compFields.draftId;
                break;
            case "mail:addressbook":
            case "mailnews:filterlist":
            case "mailnews:search":
            case "mailnews:absearch":
            case "Extension:Manager":
            case "Toolkit:PasswordManager":
            case "Mail:Preferences":
            case "Preferences:ConfigManager":
            case "Activity:Manager":
            case "Mail:About":
            case "global:console":
            case "mozapp:venkman":
            case "winInspectorMain":
                break;
            default:
                saveIt = false;
                break;
            }
            if (saveIt)
            {
                if (openWindowNum >= sessionInfo.windows.length)
                    sessionInfo.windows.push(new Object());
                siWin = sessionInfo.windows[openWindowNum];
                siWin.type = wintype;
                if (folderUri)
                    siWin.folderUri = folderUri;
                if (messageUri)
                    siWin.messageUri = messageUri;
                if (messageKey != nsMsgKey_None)
                    siWin.messageKey = messageKey;
                if (mywin == window)
                {
                    siWin.main = true;
                    sessionInfo.folderUri = folderUri;
                }
                openWindowNum++;
            }
        }
    }
    
    var JSON = Components.classes["@mozilla.org/dom/json;1"]
                         .createInstance(Components.interfaces.nsIJSON);
   
    var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                               .getService(Components.interfaces.nsIProperties);
    var sessionFile = dirService.get("ProfD", Components.interfaces.nsILocalFile);
    sessionFile.append("session.dat");

    var outStr = JSON.encode(sessionInfo);
    var stream = Components.classes["@mozilla.org/network/safe-file-output-stream;1"]
                           .createInstance(Components.interfaces.nsIFileOutputStream);

    stream.init(sessionFile, 0x02 | 0x08 | 0x20, 0600, 0);

    // convert to UTF-8
    var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"]
                              .createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    converter.charset = "UTF-8";
    var convertedData = converter.ConvertFromUnicode(outStr);
    convertedData += converter.Finish();

    // write and close stream
    stream.write(convertedData, convertedData.length);
    if (stream instanceof Components.interfaces.nsISafeOutputStream)
        stream.finish();
    else
        stream.close();
}

var gWindowSessionManager = null;
var gIsMailboxesWindow = false;
var gOpenWindowNum = 0;
var gRestoreInProgress = false;
var gOpenTabNum = 0;
var gMainWindow = null;
var gSessionInfo = null;
var gOpeningMailboxTab = false;
var gFirstSession = false;

function loadSessionInfo()
{
    // Read up the saved session file and stick in to an object
    var dirService = Components.classes["@mozilla.org/file/directory_service;1"]
                               .getService(Components.interfaces.nsIProperties);
    var sessionFile = dirService.get("ProfD", Components.interfaces.nsILocalFile);
    sessionFile.append("session.dat");
    
    if (!sessionFile.exists())
    {
        gFirstSession = true;
        return;
    }
    
    try
    {
        var stream = Components.classes["@mozilla.org/network/file-input-stream;1"].
                     createInstance(Components.interfaces.nsIFileInputStream);
        stream.init(sessionFile, 0x01, 0, 0);
        var cvstream = Components.classes["@mozilla.org/intl/converter-input-stream;1"].
                       createInstance(Components.interfaces.nsIConverterInputStream);
        cvstream.init(stream, "UTF-8", 1024, Components.interfaces.nsIConverterInputStream.DEFAULT_REPLACEMENT_CHARACTER);
        
        var content = "";
        var data = {};
        while (cvstream.readString(4096, data))
            content += data.value;
        cvstream.close();
        
        var JSON = Components.classes["@mozilla.org/dom/json;1"]
                             .createInstance(Components.interfaces.nsIJSON);
        gSessionInfo = JSON.decode(content);
    } catch (e) {}
}

function restoreTabs()
{
    if (!gOpenTabNum)
        return;
        
    if (!gSessionInfo)
        loadSessionInfo();

    if (gSessionInfo && "windows" in gSessionInfo && gSessionInfo.windows.length > gOpenWindowNum)
    {
      var siWin = gSessionInfo.windows[gOpenWindowNum];

      if ("tabs" in siWin)
      {
          for (; gOpenTabNum <= siWin.tabs.length; gOpenTabNum++)
          {
              try
              {
                  var siTab = siWin.tabs[gOpenTabNum - 1];
                  var tabFolderUri = siTab.folderUri;
                  var tabtype = siTab.type;
                  var msgFolder = GetMsgFolderFromUri(tabFolderUri, false);
                  var selectedTab = "tabSelected" in siWin && siWin.tabSelected == gOpenTabNum;
                  var args = {background: !selectedTab};

                  if (tabtype == "folder")
                      args.folder = msgFolder;
                  else
                  {
                      args.msgHdr = msgFolder.GetMessageHeader(siTab.messageKey);
                      args.forceSelectMessage = true;
                  }
                  gOpeningMailboxTab = true;
                  document.getElementById("tabmail").openTab(tabtype, args);
                  gOpeningMailboxTab = false;
                  setTimeout(restoreTabs, 0);
                  gOpenTabNum++;
                  return;
              }
              catch (e)
              {
                  dump("Exception caught restoring a tab: "+e+"\n");
              }
          }
      }
    }

    gOpenTabNum = 0;
    if (!gWindowSessionManager)
    {
        if (gMainWindow)
        {
            gMainWindow.postMessage("", "*");
            gMainWindow = null;
        }
    }
    else
    {
        gOpenWindowNum++;
        setTimeout(restoreOpenWindows, 1);
    }
}

function messageListener(event)
{
    if (!event.isTrusted || event.origin != "chrome://messenger")
        return;

    if (gWindowSessionManager)
    {
        gOpenWindowNum++;
        setTimeout(restoreOpenWindows, 1);
    }
    else
    {
        gMainWindow = window.opener;
        gOpenTabNum = 1;
        gOpenWindowNum = parseInt(event.data, 10);
        setTimeout(restoreTabs, 1000);
    }
}
window.addEventListener("message", messageListener, false);

function restoreOpenWindows()
{
    for (; gOpenWindowNum < gSessionInfo.windows.length; gOpenWindowNum++)
    {
        try
        {
            var siWin = gSessionInfo.windows[gOpenWindowNum];
            var wintype = siWin.type;

            switch (wintype)
            {
            case "mail:3pane":
                var folderUri = siWin.folderUri;

                if (siWin.main)
                {
                    window.focus();
                    setTimeout(restoreTabs, 100);
                }
                else
                    goMailbox(folderUri, false, true);
                gOpenTabNum = 1;
                return;
            case "mail:messageWindow":
                if ("messageKey" in siWin)
                {
                    var msgFolder = GetMsgFolderFromUri(siWin.folderUri, false);
                    var hdr = msgFolder.GetMessageHeader(siWin.messageKey);
                    MailUtils.openMessageInNewWindow(hdr);
                }
                break;
            case "msgcompose":
                var messageUri = siWin.messageUri;
                ComposeMessage(Components.interfaces.nsIMsgCompType.Draft,
                               Components.interfaces.nsIMsgCompFormat.Default, 
                               null, [messageUri]);
                break;
            case "mail:addressbook":
                toOpenWindowByType("mail:addressbook", "chrome://messenger/content/addressbook/addressbook.xul");
                break;
            case "mailnews:filterlist":
                var args = { refresh: true };
                MsgFilterList(args);
                break;
            case "mailnews:search":
                MsgSearchMessages();
                break;
            case "mailnews:absearch": 
                MsgSearchAddresses();
                break;
            case "Extension:Manager":
                openAddonsMgr();
                break;
            case "Toolkit:PasswordManager":
                toOpenWindowByType("Toolkit:PasswordManager", "chrome://messenger/content/preferences/viewpasswords.xul");
                break;
            case "Mail:Preferences":
                openOptionsDialog();
                break;
            case "Preferences:ConfigManager":
                toOpenWindowByType("Preferences:ConfigManager", "chrome://global/content/config.xul");
                break;
            case "Activity:Manager":
                openActivityMgr();
                break;
            case "Mail:About":
                openAboutDialog();
                break;
            case "global:console":
                toJavaScriptConsole();
                break;
            case "mozapp:venkman":
                start_venkman();
                break;
            case "winInspectorMain":
                inspectDOMDocument();
                break;
            default:
                continue;
            }

            // When the event gets fired after the window is opened up, it will
            // call this function again to open up the rest of the windows
            gOpenWindowNum++;
            return;
        }
        catch (e)
        {
            dump("Exception caught restoring a window: "+e+"\n");
        }
    }

    gRestoreInProgress = false;
}

function windowSessionManager()
{
    var obsSvc = Components.classes["@mozilla.org/observer-service;1"]
                           .getService(Components.interfaces.nsIObserverService);

    // Get notified when we first start up so that previous session windows can be restored
    obsSvc.addObserver(this, "mail-startup-done", false);

    // Get notified when we are about to shut down, but before windows start closing
    obsSvc.addObserver(this, "quit-application-granted", false);

    // Register the listener for window changes now that we've opened up
    // all the windows from the previous session and that will no longer
    // cause spurious event firings to the listener
    var mediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                             .getService(Components.interfaces.nsIWindowMediator);
    mediator.addListener(this);

    loadSessionInfo();
}

function continueRestoring(evt)
{
    try
    {
        evt.target.removeEventListener("load", continueRestoring, false);
        if (gOpenTabNum)
            evt.target.defaultView.postMessage(gOpenWindowNum, "*");
        else
            setTimeout(restoreOpenWindows, 1);
    }
    catch(e)
    {
    }
}

windowSessionManager.prototype = 
{
    // nsIWindowMediatorListener:
    onOpenWindow:        function(win)
    {
        if (gRestoreInProgress)
        {
            if (win.docShell)
            {
                var requestor = win.docShell.QueryInterface(Components.interfaces.nsIInterfaceRequestor);
                var mywin = requestor.getInterface(Components.interfaces.nsIDOMWindow);
                if (mywin != window)
                    mywin.addEventListener("load", continueRestoring, false);
            }
            return;
        }

        // Can't call the routine to save open windows directly because it uses the window mediator,
        // and that will will cause a crash because we're in the middle of a window mediator
        // listener notification.
        // Also, when opening windows there has to be a pause to wait for the window to have
        // enough time to bring itself up and be fully registered with the window mediator.
        setTimeout(saveOpenWindows, 500);
    },
    onCloseWindow:       function(win) { setTimeout(saveOpenWindows, 1); },
    onWindowTitleChange: function(win, newTitle) {},
    
    // nsIObserver
    observe:             function(subject, topic, data)
    {
        var obsSvc = Components.classes["@mozilla.org/observer-service;1"]
                               .getService(Components.interfaces.nsIObserverService);
        switch (topic)
        {
        case "mail-startup-done":
            obsSvc.removeObserver(this, "mail-startup-done");

            try
            {
                // FORNOW, Get rid of TB's session file because it interferes with ours
                var file = Components.classes["@mozilla.org/file/directory_service;1"]
                                     .getService(Components.interfaces.nsIProperties)
                                     .get("ProfD", Components.interfaces.nsIFile);
                file.append("session.json");
                if (file.exists())
                    file.remove(false);
            }
            catch(e) {}

            // Now that the main window is loaded up it's time to start
            // restoring windows from a previous session.  Doing it in the
            // middle of the notification will cause Bad Things (TM) to happen.
            if (gPrefBranch.getBoolPref("penelope.sessionstore.restoreOnStartup") && gSessionInfo)
            {                
                gRestoreInProgress = true;
                setTimeout(restoreOpenWindows, 1);
            }
            break;
            
        case "quit-application-granted":
            obsSvc.removeObserver(this, "quit-application-granted");

            if (gWindowSessionManager)
            {
                // Save the open windows one last time before quitting the app,
                // just to make sure we've saved all the windows and the z-order.
                saveOpenWindows(true);
                
                var mediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                                         .getService(Components.interfaces.nsIWindowMediator);
                mediator.removeListener(gWindowSessionManager);
                gWindowSessionManager = null;
            }
            break;
        default:
            break;
        }
    }
};

var gAutoSelectHdr = null;

window.originalMarkMessageAsRead = window.MarkMessageAsRead;
window.MarkMessageAsRead = penelopeMarkMessageAsRead;

function penelopeMarkMessageAsRead(msgHdr)
{
    // Don't auto-mark as read preview messages immediately as that's
    // not what Classic Eudora did and it's just a bad idea because
    // just arrowing through a mailbox will mark each message as read.
    // If the user really wants to do it they can set it to mark as
    // read on a delay and set the delay to 0 seconds.
    if ((!gPrefBranch.getBoolPref("mailnews.mark_message_read.auto") ||
          gPrefBranch.getBoolPref("mailnews.mark_message_read.delay") ||
          gFolderDisplay._tabInfo.mode.name != "folder") &&
         gAutoSelectHdr != msgHdr)
    {
        window.originalMarkMessageAsRead(msgHdr);
    }
    gAutoSelectHdr = null;
}

function SelectFUMLUB(msgFolder, dontFocusTree)
{
    if ("ignoreNextFUMLUB" in window)
        delete window.ignoreNextFUMLUB;
    else
    {
        // We need to delay the FUMLUB selection because the core code winds up
        // scrolling the thread pane and we want to do FUMLUB after that
        setTimeout(DelayedSelectFUMLUB, 1, msgFolder, dontFocusTree);
    }
}

function DelayedSelectFUMLUB(msgFolder, dontFocusTree)
{
    // There's only one view per mailbox window, so if we're trying to do FUMLUB
    // selection on the non-current tab then set a flag so that we do it once
    // the tab comes to the foreground
    var tabmail = document.getElementById('tabmail');
    var tabInfo = tabmail.currentTabInfo;

    // Only do FUMLUB for folders, not any other types, and more specifically not messages
    if (tabInfo.mode.type != "folder")
        return;

    if (tabInfo.folderDisplay.displayedFolder != msgFolder)
    {
        var numTabs = tabmail.tabInfo.length;
        var tabIndex;
        for (tabIndex = 0; tabIndex < numTabs; tabIndex++)
        {
            tabInfo = tabmail.tabInfo[tabIndex];
            if (tabInfo.mode.name == "folder" && msgFolder == tabInfo.folderDisplay.displayedFolder)
            {
                tabInfo.needsFUMLUBSelection = true;
                break;
            }
        }
        return;
    }

    const numItems = gDBView.rowCount;

    if (!numItems)
        return;

    // Default to selecting the first or last item in the mailbox in case we don't do FUMLUB,
    // which happens if the pref is off or the mailbox isn't sorted a way that puts newly
    // inserted messages at one end or the other
    const sortType = gDBView.sortType;
    const timeOrientedSort = sortType == nsMsgViewSortType.byDate ||
                             sortType == nsMsgViewSortType.byReceived;
    const sortOrder = gDBView.sortOrder;
    var row, inc;

    if (timeOrientedSort ^ sortOrder == nsMsgViewSortOrder.ascending)
    {
        row = 0;
        inc = 1;
    }
    else
    {
        row = numItems - 1;
        inc = -1;
    }

    var selectKey = gDBView.getKeyAt(row);
    var selectHdr = gDBView.db.GetMsgHdrForKey(selectKey);

    if (gPrefBranch.getBoolPref("penelope.mailbox.selectFUMLUB"))
    {
        // Only do FUMLUB selection if the mailbox is sorted by Date, Received, None, or Custom (which will be group-select)
        if (sortType == nsMsgViewSortType.byDate ||
            sortType == nsMsgViewSortType.byReceived ||
            sortType == nsMsgViewSortType.byNone ||
            sortType == nsMsgViewSortType.byCustom)
        {
            selectHdr = null;
            for (; row >= 0 && row < numItems; row += inc)
            {
                var msgKey = gDBView.getKeyAt(row);
                var hdr = gDBView.db.GetMsgHdrForKey(msgKey);
                if (hdr.isRead)
                {
                    // If the first message isn't unread, then use it as the selection
                    if (!selectHdr)
                    {
                        selectKey = msgKey;
                        selectHdr = hdr;
                    }
                    break;
                }
                else
                {
                    selectKey = msgKey;
                    selectHdr = hdr;
                }
            }
        }
    }

    if (selectKey != -1)
    {
        // Make sure the most unread messages in the block are shown by first
        // scrolling to the end of the mailbox we started looking at
        var tree = GetThreadTree();
        tree.treeBoxObject.ensureRowIsVisible(inc > 0? 0 : numItems - 1); 

        // We want to select the message so that the message contents get shown in
        // the preview pane, but we don't want the message to get marked as read so
        // we set a flag and ignore the next time we get a request to mark as read
        gAutoSelectHdr = selectHdr;
        gDBView.selectMsgByKey(selectKey);
        
        // Finally, give the thread tree the focus
        if (!dontFocusTree)
            tree.focus();
    }
}
 
function delayedOnLoadPenelopeMessenger()
{
    // Add a listener for handling double-click and Enter key on a folder
    var folderTree = document.getElementById("folderTree"); 
    folderTree.addEventListener("click",FolderTreeOnClick,true);
    folderTree.addEventListener("keypress",FolderTreeOnKeyPress,true);
    
    // Add a listener for handling Alt+click
    var threadTree = document.getElementById("threadTree"); 
    threadTree.addEventListener("mousedown",ThreadTreeOnMouseDown,true);
    threadTree.addEventListener("click",ThreadTreeOnMouseClick,true);
    threadTree.addEventListener("keypress",ThreadTreeOnKeyPress,true);
    threadTree.addEventListener("keyup",ThreadTreeOnKeyUp,true);
    
    updateMailboxFont();
    updateShowMailbox();
    gPrefBranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
    gPrefBranch.addObserver("penelope.mailbox.", mailboxPrefObserver, false);
    updateMailboxCss();
    enableMailboxLines();
    updateIconColumns();
}

var mailboxPrefObserver = 
{
    observe: function(subject, topic, data)
    {
        switch (data)
        {
        case "penelope.mailbox.font.name":
        case "penelope.mailbox.font.size":
            updateMailboxFont();
            break;
        case "penelope.mailbox.showPreviewPane":
            if (!gIsMailboxesWindow)
                updatePreviewPane();
            break;
        case "penelope.mailbox.selectShowsMailbox":
            updateShowMailbox();
            break;
        case "penelope.mailbox.fixIconColumns":
        case "penelope.mailbox.fixIconColumns.width":
            updateIconColumns();
            break;
        default:
            break;
        }
    }
};

function updateMailboxFont()
{
    var styleSheet = document.styleSheets[document.styleSheets.length - 1];
    var ruleList = styleSheet.cssRules;
    const ruleStart = "#mailContent {";

    // Remove the previous mailbox font style sheet, if found
    var re = new RegExp("^" + ruleStart);
    var ruleIndex;
    for (ruleIndex = 0; ruleIndex < ruleList.length; ruleIndex++)
    {
        if (ruleList[ruleIndex].cssText.match(re))
            styleSheet.deleteRule(ruleIndex);
    }

    // Change the stylesheet with the mailbox font prefs
    try
    {
        var fontName = gPrefBranch.getCharPref("penelope.mailbox.font.name");
        var fontSize = gPrefBranch.getIntPref("penelope.mailbox.font.size");
        if (fontName.length || fontSize)
        {
            var rule = ruleStart;
            if (fontName.length)
                rule += "font-family:"+ fontName +" !important;";
            if (fontSize)
                rule += "font-size:" + fontSize + "pt !important;";
            rule += "}";
            styleSheet.insertRule(rule, ruleList.length);
        }
    }
    catch(e)
    {
        dump("Error setting mailbox font: "+e+"\n");
    }
}

function updateShowMailbox()
{
    var menuItem = document.getElementById('folderPaneContext-showMailbox');
    var hide = true;
    
    if (!gIsMailboxesWindow)
        hide = gPrefBranch.getBoolPref("penelope.mailbox.selectShowsMailbox");
    menuItem.hidden = hide;
}

var newMailListener =
{
    msgsClassified: function(aMsgHdrs)
    {
        if (gPrefBranch.getBoolPref("penelope.mailbox.autoOpen"))
        {
            const nsMsgFolderFlags = Components.interfaces.nsMsgFolderFlags;
            var uriArray = [];
            for each (var msgHdr in fixIterator(aMsgHdrs.enumerate(), Components.interfaces.nsIMsgDBHdr))
            {
                const folder = msgHdr.folder;
                if (!folder.isSpecialFolder(nsMsgFolderFlags.Trash |
                                            nsMsgFolderFlags.Junk |
                                            nsMsgFolderFlags.SentMail |
                                            nsMsgFolderFlags.Drafts |
                                            nsMsgFolderFlags.Queue,
                                            false))
                {
                    const URI = folder.URI;
                    if (uriArray.indexOf(URI) < 0)
                        uriArray.push(URI);
                }
            }
            for (var i = 0; i < uriArray.length; i++)
                goMailbox(uriArray[i], false, false, false, true);
        }
    }
};

var penelopeFolderListener =
{
    OnItemAdded: function(parentItem, item) { },
    OnItemRemoved: function(parentItem, item) { },
    OnItemPropertyChanged: function(item, property, oldValue, newValue) { },
    OnItemIntPropertyChanged: function(item, property, oldValue, newValue) { },
    OnItemBoolPropertyChanged: function(item, property, oldValue, newValue) { },
    OnItemUnicharPropertyChanged: function(item, property, oldValue, newValue) { },
    OnItemPropertyFlagChanged: function(item, property, oldFlag, newFlag) { },

    OnItemEvent: function(folder, event)
    {
        if (event == "FolderLoaded")
        {
            // Only do FUMLUB selection if the folder hasn't previously loaded
            // a message.  For some reason the FolderLoaded event gets sent at
            // other times, even not when the folder is being loaded.
            if (folder.lastMessageLoaded == nsMsgKey_None)
                SelectFUMLUB(folder, true);
        }
    }
};

function penelopeMessengerOnLoad()
{
    dump("In penelopeMessengerOnLoad\n");

    // Create a window session manager for only the very first mailbox window
    var mediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
                             .getService(Components.interfaces.nsIWindowMediator);
    var windows = mediator.getEnumerator("mail:3pane");
    if (!windows.hasMoreElements() || !windows.getNext() || !windows.hasMoreElements())
    {
        var notificationService = Components.classes["@mozilla.org/messenger/msgnotificationservice;1"]
                                            .getService(Components.interfaces.nsIMsgFolderNotificationService);
        notificationService.addListener(newMailListener, notificationService.msgsClassified);
        
        gWindowSessionManager = new windowSessionManager;
    }

    var mailSession = Components.classes["@mozilla.org/messenger/services/session;1"]
                                .getService(Components.interfaces.nsIMsgMailSession);
    var notifyFlags = Components.interfaces.nsIFolderListener.event;
    mailSession.AddFolderListener(penelopeFolderListener, notifyFlags);

    var uri = null;
    if (window.arguments.length)
    {
        // filter our any feed urls that came in as arguments to the new window...
        if (/^feed:/i.test(window.arguments[0]))
        {
            // Leave uri=null
        }
        else
        {
            uri = window.arguments[0];
            if (!uri && gSessionInfo && gSessionInfo.folderUri)
                uri = gSessionInfo.folderUri;
            if (!uri)
            {
                var msgFolder = GetDefaultAccountRootFolder();
                if (msgFolder)
                    uri = msgFolder.URI;
                else
                    gFirstSession = true;
            }
            window.arguments[0] = uri;
            dump("Folder URI = "+uri+"\n");
        }
    }

    var use3PaneUI = gPrefBranch.getBoolPref("penelope.ui3pane");

    if (!use3PaneUI && gWindowSessionManager)
        gIsMailboxesWindow = true;
        
    // We call these initialization functions both now and after the
    // OnLoadMessenger() call.  This gets the right window
    // size/location/visible panes in the end and prevents redraw
    // flicker while it happens.
    if (!gIsMailboxesWindow)
    {
        initializeFolderWin(uri, use3PaneUI);
        setTimeout(initializeFolderWin, 0, uri, use3PaneUI);
    }
    else
    {
        // This is the 2-pane folder summary window
        initializeSummaryFolderWin();
        setTimeout(initializeSummaryFolderWin, 0);
    }

    OnLoadMessenger();

    if (!use3PaneUI && !gIsMailboxesWindow)
    {
        var tabmail = document.getElementById('tabmail');
        tabmail.tabInfo[0].mode.legalPanes.folder = false;
    }

    // Remove some menu items for Mac
    try
    {
        if (navigator.platform.match("^Mac"))
        {
            var toolsMenu = document.getElementById('taskPopup');
            toolsMenu.removeChild(toolsMenu.getElementsByAttribute('id', 'tasksMenuMail')[0]);
            toolsMenu.removeChild(toolsMenu.getElementsByAttribute('id', 'addressBook')[0]);
        }
    }
    catch(e)
    {
    }

    // This is a one-shot init of the folder pane to "All Folders" view
    // since its most like Classic Eudora and the default is "Smart Folders"
    if (gFirstSession)
    {
        try
        {
            gFolderTreeView.mode = "all";
        } catch(e){}
    }
}

window.originalLoadPostAccountWizard = window.LoadPostAccountWizard;
window.LoadPostAccountWizard = penelopeLoadPostAccountWizard;

function penelopeLoadPostAccountWizard()
{
    // Need to set the folder pane view mode to "all" once more as
    // the first time it gets set the tree isn't fully initialized
    window.originalLoadPostAccountWizard();
    if (gFirstSession)
    {
        gFolderTreeView.mode = "all";
        
        var msgFolder = GetDefaultAccountRootFolder();
        var index = gFolderTreeView.getIndexOfFolder(msgFolder);
        if (index >= 0)
        {
            gFolderTreeView.selection.select(index);
            gFolderTreeView.toggleOpenState(index);
        }
    }
}

function hideItem(id)
{
    var item = document.getElementById(id);
    if (item)
        item.hidden = true;
}

function disableItem(id)
{
    var item = document.getElementById(id);
    if (item)
        item.disabled = true;
}

function initializeSummaryFolderWin()
{
    // reconfigure for 2-pane mode folder window
    var toolbar = document.getElementById('mail-bar3');
    toolbar.collapsed = true;

    // Add flex to the folder pane
    document.getElementById('folderPaneBox').setAttribute('flex', '1');

    // Remove flex from vbox with no ID, and hide the splitter
    document.getElementById('folderpane_splitter').nextSibling.removeAttribute('flex');
    document.getElementById('folderpane_splitter').nextSibling.collapsed = true;
    document.getElementById("folderpane_splitter").hidden = true;

    document.getElementById('threadPaneBox').removeAttribute('flex');
    document.getElementById('threadTree').removeAttribute('flex');
    document.getElementById('threadTree').setAttribute("flags", "dont-build-content");
    document.getElementById('threadTree').disabled=true;

    //var splitter = document.getElementById('threadpane-splitter');
    //splitter.collapsed = false;
    document.documentElement.setAttribute("width", 300);
    document.documentElement.setAttribute("height", 700);
    
    // Make sure tab is always hidden, even if mail.tabs.autoHide pref is off
    document.getElementById("tabmail").tabContainer.mAutoHide = true;
    
    // Disable the context menu that opens folders in tabs    
    document.getElementById("folderPaneContext-openNewTab").disabled = true;

    // Hide/disable some menus and menu items that never work for the Mailboxes window
    hideItem('CustomizeMailToolbar');
    
    // Top level menus
    hideItem('menu_Go');
    hideItem('messageMenu');
    hideItem('penelopeTransferMenu');
    
    // File menu
    hideItem('fileAttachmentMenu');
    hideItem('menu_saveAs');
    hideItem('fileMenuAfterSaveSeparator');
    disableItem('printSetupMenuItem');
    disableItem('printPreviewMenuItem');
    disableItem('printMenuItem');
    
    // Edit menu
    hideItem('menu_select');
    hideItem('editMenuAfterSelectSeparator');
    hideItem('penelopeSelectedTextUrlSeparator');
    hideItem('penelopeDoSelectedTextUrl1');
    hideItem('penelopeDoSelectedTextUrl2');
    hideItem('penelopeDoSelectedTextUrl3');
    hideItem('penelopeDoSelectedTextUrl4');
    hideItem('penelopeDoSelectedTextUrl5');
    hideItem('penelopeDoSelectedTextUrl6');

    // View menu
    hideItem('menu_Toolbars');
    hideItem('menu_MessagePaneLayout');
    hideItem('viewSortMenuSeparator');
    hideItem('viewSortMenu');
    hideItem('viewMessagesMenu');
    hideItem('viewheadersmenu');
    hideItem('viewAfterThreadsSeparator');
    hideItem('viewMessageViewMenu');
    hideItem('viewBodyMenu');
    hideItem('viewAttachmentsInlineMenuitem');
    hideItem('viewAfterAttachmentsSeparator');
    hideItem('viewFullZoomMenu');
    hideItem('mailviewCharsetMenu');
    hideItem('viewAfterCharsetSeparator');
    hideItem('pageSourceMenuItem');
    hideItem('menu_securityStatus');

    // Help menu
    hideItem('whatsNew');
    hideItem('featureConfigurator');

    try
    {
        dump("Folder Window Dimentions:\n");
        var width =   gPrefBranch.getIntPref("penelope.__folderwindow.width");
        var height =  gPrefBranch.getIntPref("penelope.__folderwindow.height");
        var screenX = gPrefBranch.getIntPref("penelope.__folderwindow.screenX");
        var screenY = gPrefBranch.getIntPref("penelope.__folderwindow.screenY");

        // Have to both call the function and set the attributes in order
        // to prevent redraw flicker when the window is being opened up
        dump("Moving to: "+screenX+", "+screenY+"\n");
        window.moveTo(screenX, screenY);
        document.documentElement.setAttribute("screenX", screenX);
        document.documentElement.setAttribute("screenY", screenY);

        dump("Resize to: "+width+", "+height+"\n");
        window.resizeTo(width, height);
        document.documentElement.setAttribute("width", width);
        document.documentElement.setAttribute("height", height);
    }
    catch(e)
    {
        dump("Error setting folder pane attributes: "+e+"\n");
    }

    try
    {
        var windowState = gPrefBranch.getIntPref("penelope.__folderwindow.windowState");
        if (windowState == window.STATE_MINIMIZED)
            window.minimize();
        else if (windowState == window.STATE_MAXIMIZED)
            window.maximize();
    }catch(e){}
}

function updatePreviewPane()
{
    var showPreviewPane = gPrefBranch.getBoolPref("penelope.mailbox.showPreviewPane");
    
    if (gMessageDisplay && gMessageDisplay.visible != showPreviewPane)
    {
       MsgToggleMessagePane();
       if (!gMessageDisplay._active)
           gMessageDisplay._visible = showPreviewPane;
    }
}

// Init 2-pane or 3-pane window depending on the UI mode
function initializeFolderWin(uri, use3PaneUI)
{
    if (uri)
    {
        try
        {
            try
            {
                dump("FolderURI: "+uri+"\n");
                var width =   gPrefBranch.getIntPref("penelope."+uri+".window.width");
                var height =  gPrefBranch.getIntPref("penelope."+uri+".window.height");
                var screenX = gPrefBranch.getIntPref("penelope."+uri+".window.screenX");
                var screenY = gPrefBranch.getIntPref("penelope."+uri+".window.screenY");

                // Have to both call the function and set the attributes in order
                // to prevent redraw flicker when the window is being opened up
                dump("Moving to: "+screenX+", "+screenY+"\n");
                window.moveTo(screenX, screenY);
                document.documentElement.setAttribute("screenX", screenX);
                document.documentElement.setAttribute("screenY", screenY);

                dump("Resize to: "+width+", "+height+"\n");
                window.resizeTo(width, height);
                document.documentElement.setAttribute("width", width);
                document.documentElement.setAttribute("height", height);
            }catch(e){}

            try
            {
                var windowState = gPrefBranch.getIntPref("penelope."+uri+".window.windowState");
                if (windowState == window.STATE_MINIMIZED)
                    window.minimize();
                else if (windowState == window.STATE_MAXIMIZED)
                    window.maximize();
            }catch(e){}

            var folderPane = document.getElementById("folderPaneBox"); 
            if (use3PaneUI)
            {
                var fpcollapsed = gPrefBranch.getBoolPref("penelope."+uri+".window.fpcollapsed");
                var fpwidth = gPrefBranch.getIntPref("penelope."+uri+".window.fpwidth");

                var folderPaneSplitter = document.getElementById("folderpane_splitter"); 
                var fpsplitter = gPrefBranch.getCharPref("penelope."+uri+".window.fpsplitter");

                dump("Setting fpwidth = "+fpwidth+"\n");
                dump("Setting fpcollapsed = "+fpcollapsed+"\n");
                dump("Setting fpsplitter = "+fpsplitter+"\n");
                if (fpwidth)
                {
                    folderPane.width = fpwidth;
                }

                folderPane.collapsed = fpcollapsed;
                if (fpsplitter)
                {
                    folderPaneSplitter.setAttribute("state", fpsplitter);
                }
            }
        }
        catch(e)
        {
            dump("Error setting folder pane attributes: "+e+"\n");
        }

        try
        {
            var messagePane = document.getElementById("messagepanebox"); 
            var mpwidth = gPrefBranch.getIntPref("penelope."+uri+".window.mpwidth");
            var mpheight = gPrefBranch.getIntPref("penelope."+uri+".window.mpheight");

            var threadPaneSplitter = document.getElementById("threadpane-splitter"); 
            var tpsplitter = gPrefBranch.getCharPref("penelope."+uri+".window.tpsplitter");

            dump("Setting tpsplitter = "+tpsplitter+"\n");
            dump("Setting mpwidth = "+mpwidth+"\n");
            dump("Setting mpheight = "+mpheight+"\n");
            if (mpwidth)
            {
                messagePane.width = mpwidth;
            }
            if (mpheight)
            {
                messagePane.height = mpheight;
            }
            if (tpsplitter)
            {
                threadPaneSplitter.setAttribute("state", tpsplitter);
            }
        }
        catch(e)
        {
            dump("Error setting thread pane attributes: "+e+"\n");
        }

        updatePreviewPane();
    }
}

function penelopeMessengerOnUnload()
{
    dump("In penelopeMessengerOnUnload\n");
    
    // See if this window is the main folder window
    if (!gIsMailboxesWindow)
    {
        // Save off the window state info of the mailbox in the first tab
        var tabmail = document.getElementById('tabmail');
        var tabInfo = tabmail.tabInfo[0];
        var folderDisplay = tabInfo.folderDisplay;
        var uri = folderDisplay.displayedFolder.URI;

        // Set the preferences for the window instance
        gPrefBranch.setIntPref("penelope."+uri+".window.windowState", window.windowState);
        if (window.windowState == window.STATE_NORMAL)
        {
            dump("Saving dimentions as: "+window.outerWidth+", "+window.outerHeight+"\n");
            dump("Saving position as: "+window.screenX+", "+window.screenY+"\n");
            gPrefBranch.setIntPref("penelope."+uri+".window.width", window.outerWidth);
            gPrefBranch.setIntPref("penelope."+uri+".window.height", window.outerHeight);
            gPrefBranch.setIntPref("penelope."+uri+".window.screenX", window.screenX);
            gPrefBranch.setIntPref("penelope."+uri+".window.screenY", window.screenY);
        }

        // Only store folder pane settings if in 3 pane mode
        if (gPrefBranch.getBoolPref("penelope.ui3pane"))
        {
            try
            {
                var folderPane = document.getElementById("folderPaneBox");
                var fpcollapsed = !folderDisplay.folderPaneVisible;
                var fpwidth = folderPane.width;
                var fpsplitter = fpcollapsed? "collapsed" : "";

                dump("Saving fpcollapsed = "+fpcollapsed+"\n");
                gPrefBranch.setBoolPref("penelope."+uri+".window.fpcollapsed", fpcollapsed);
                dump("Saving fpwidth = "+fpwidth+"\n");
                gPrefBranch.setIntPref("penelope."+uri+".window.fpwidth", fpwidth);
                dump("Saving fpsplitter = "+fpsplitter+"\n");
                gPrefBranch.setCharPref("penelope."+uri+".window.fpsplitter", fpsplitter);

            }
            catch(e)
            {
                dump("Caught folder pane exception: "+e+"\n");
            }
        }

        // Only store thread pane settings if showing the preview pane
        if (gPrefBranch.getBoolPref("penelope.mailbox.showPreviewPane"))
        {
            try
            {
                var messagePane = document.getElementById("messagepanebox");
                var mpwidth = messagePane.width;
                var mpheight = messagePane.height;
                var tpsplitter = folderDisplay.messagePaneCollapsed? "collapsed" : "";

                dump("Saving tpsplitter = "+tpsplitter+"\n");
                gPrefBranch.setCharPref("penelope."+uri+".window.tpsplitter", tpsplitter);
                dump("Saving mpwidth = "+mpwidth+"\n");
                gPrefBranch.setIntPref("penelope."+uri+".window.mpwidth", mpwidth);
                dump("Saving mpheight = "+mpheight+"\n");
                gPrefBranch.setIntPref("penelope."+uri+".window.mpheight", mpheight);
            }
            catch(e)
            {
                dump("Caught thread pane exception: "+e+"\n");
            }
        }
    }
    else
    {
        try
        {
            // Set the preferences for the folder window 
            gPrefBranch.setIntPref("penelope.__folderwindow.windowState", window.windowState);
            if (window.windowState == window.STATE_NORMAL)
            {
                dump("Saving dimentions as: "+window.outerWidth+", "+window.outerHeight+"\n");
                dump("Saving position as: "+window.screenX+", "+window.screenY+"\n");
                gPrefBranch.setIntPref("penelope.__folderwindow.width", window.outerWidth);
                gPrefBranch.setIntPref("penelope.__folderwindow.height", window.outerHeight);
                gPrefBranch.setIntPref("penelope.__folderwindow.screenX", window.screenX);
                gPrefBranch.setIntPref("penelope.__folderwindow.screenY", window.screenY);
            }
        }
        catch(e)
        {
            dump("Exception: In folder win - "+ e+"\n");
        }
    }
    
    var notificationService = Components.classes["@mozilla.org/messenger/msgnotificationservice;1"]
                                        .getService(Components.interfaces.nsIMsgFolderNotificationService);
    notificationService.removeListener(newMailListener); 

    var mailSession = Components.classes["@mozilla.org/messenger/services/session;1"]
                                .getService(Components.interfaces.nsIMsgMailSession);
    mailSession.RemoveFolderListener(penelopeFolderListener);

    gPrefBranch.QueryInterface(Components.interfaces.nsIPrefBranch2);
    gPrefBranch.removeObserver("penelope.mailbox.", mailboxPrefObserver);

    OnUnloadMessenger();
}

function penelopeCanClose()
{
    if (gWindowSessionManager)
    {
        if (gPrefBranch.getBoolPref("penelope.noCloseMainWindow"))
            return false;

        // If we're closing the window with the session manager (the main window)
        // and the pref is turned on, then quit
        if (gPrefBranch.getBoolPref("penelope.closeMainWindowQuits"))
        {
            setTimeout(goQuitApplication, 1);

            // Tell the window not to close since we want to save this window as part of the
            // list of open windows, but it will close in response to the quit
            return false;
        }

        saveOpenWindows(true);
    }
    
    return true;
}

function penelopeCmdClose()
{
    // Don't call penelopeCanClose() when there's more than one tab
    // because it can wind up shutting down the entire app
    var tabmail = document.getElementById('tabmail');
    if (tabmail.tabInfo.length > 1 || penelopeCanClose())
        CloseTabOrWindow();
}

var gFolderTree;

function GetFolderTree()
{
    if (!gFolderTree)
        gFolderTree = document.getElementById("folderTree");
    return gFolderTree;
}

function GetSelectedFolderURI()
{
    var selectedFolders = GetSelectedMsgFolders();

    if (selectedFolders && selectedFolders[0] && selectedFolders[0].URI)
        return selectedFolders[0].URI;

    return null;
}

// This is an override of the function in mailWindowOverlay.js so that we can
// call our own open mailbox routine that's smarter about how to open them
window.MsgOpenNewTabForFolder = penelopeMsgOpenNewTabForFolder;
function penelopeMsgOpenNewTabForFolder(aBackground)
{
    var selectedFolders = gFolderTreeView.getSelectedFolders();
    for (var i = 0; i < selectedFolders.length; i++)
        goMailbox(selectedFolders[i].URI, true, false, true);
}

function FolderTreeOnClick(event)
{
    // we only care about button 0 (left click) events, and double-click
    //
    // We have to catch this during the single click event because otherwise
    // the mailnews code will get the single click event before we get the
    // double-click event, and we need to prevent further handling of the event.
    if (event.button !== 0 || event.detail !== 2)
        return;

    var folderTree = GetFolderTree();
    var row = {};
    var col = {};
    var elt = {};
    folderTree.treeBoxObject.getCellAt(event.clientX, event.clientY, row, col, elt);
    if (row.value >= 0 && elt.value && elt.value != "twisty")
        FolderTreeDoubleClick(row.value, event);
}

function FolderTreeDoubleClick(folderIndex, event)
{
    // Open a new msg window only if we are double clicking on
    // folders or newsgroups, not top-level servers
    var uri = GetSelectedFolderURI();
    if (uri)
    {
        var view = GetFolderTree().view;
        var level = view.getLevel(folderIndex);
        var isContainer = view.isContainer(folderIndex);
        var isEmpty = view.isContainerEmpty(folderIndex);
        
        if (level > 0 || !isContainer || isEmpty)
        {
            goMailbox(uri, true, false);
            event.stopPropagation();
        }
    }
}

function FolderTreeOnKeyPress(event)
{
    if (event.keyCode == event.DOM_VK_ENTER || event.keyCode == event.DOM_VK_RETURN)
    {
        var folderTree = GetFolderTree();
        FolderTreeDoubleClick(folderTree.currentIndex, event);
    }
}

var eatNextMouseClick = false;
var eatNextAltUp = false;

function ThreadTreeOnMouseDown(event)
{
    // We only care about left mouse buttons clicks
    if (event.button !== 0)
        return;

    var threadTree = document.getElementById("threadTree");
    var treeBoxObj = threadTree.treeBoxObject;
    var row = {}, col= {}, type = {};
    treeBoxObj.getCellAt(event.clientX, event.clientY, row, col, type);

    // Only continue if the click was on a valid row
    if (row.value < 0)
        return;

    // See if user Alt+clicked on a message summary
    if (event.altKey)
    {
        MailboxGroupSelect(threadTree, row.value, col.value, !event.shiftKey);

        // Don't let the mouse down continue as it will cause the
        // clicked item in some columns (e.g. Starred and Junk) to
        // change state
        event.stopPropagation();
        // Also, we need to eat the resulting mouse click as otherwise
        // it will cause the clicked item to only be selected after
        // we've done all of the special selection work
        eatNextMouseClick = true;
        // Similar thing for the Alt up key event as otherwise it will
        // cause the menu to get the focus
        eatNextAltUp = true;
    }
    else
    {
        var id = col.value.id;
        // See if the user left-clicked in the Junk, Starred or Read columns
        // If associated prefs are set, do not propagate the click
        // This will prevent the click from accidentally Junking, Starring, Marking as Read a message
        // Also check for cycler columns that don't actually cycle
        if ((id == "junkStatusCol" && gPrefBranch.getBoolPref("penelope.mailbox.ignoreJunkColumnClick")) ||
            (id == "flaggedCol" && gPrefBranch.getBoolPref("penelope.mailbox.ignoreStarColumnClick")) ||
            (id == "unreadButtonColHeader" && gPrefBranch.getBoolPref("penelope.mailbox.ignoreReadColumnClick")) ||
            id == "statusCol" ||
            id == "priorityCol" ||
            id == "attachmentCol")
            
        {
            event.stopPropagation();
            threadTree.view.selection.select(row.value);
        }
    }
}

function ThreadTreeOnMouseClick(event)
{
    if (event.button === 0 && eatNextMouseClick)
    {
        eatNextMouseClick = false;
        event.stopPropagation();
    }
}

var lastKeyPress = 0;
var ttsBuffer;

function ThreadTreeOnKeyPress(event)
{
    if (!event.ctrlKey && !event.altKey && !event.metaKey)
    {
        var charCode = event.charCode;
        
        if (charCode >= 32)
        {
            var now = Date.now();
            var findAgain = false;
            
            // Start search criteria from the beginning if it's
            // been more than a second since a key was pressed
            if (now > lastKeyPress + 1000)
            {
                // Period key after timeout is the "find again" hotkey
                //
                // Yeah, yeah, yeah, Shift+<period> is not necessarily the greater-than character (62)
                // on all keyboards, but key presses don't give you the key code and I don't feel like
                // writing an additional key down event handler for such a minor thing.  ;^>
                if (charCode == 46 || (event.shiftKey && charCode == 62 && ttsBuffer))
                    findAgain = true;
                else
                {
                    // If this is a leading space then don't process it as we want it
			        // go through the standard space bar paging handler.
                    if (charCode == 32)
                        return;

                    ttsBuffer = "";
                }
            }

            event.preventDefault();
            event.stopPropagation();
            
            if (!findAgain)
            {
                lastKeyPress = now;
                
                // Encode the characters so as to not to be interpreted as RegEx specials (e.g. '*')
                ttsBuffer += "\\x" + charCode.toString(16);
            }
            
            var tree = document.getElementById("threadTree"); 
            var view = tree.view;
            var treeBoxObj = tree.treeBoxObject;
            var selection = view.selection;
            var numItems = view.rowCount;
            var subjectCol = treeBoxObj.columns.getNamedColumn("subjectCol");

            // If the mailbox is ascending sorted then search from top to bottom.
            // If descending or not sorted, then go bottom to top.
            // Reverse it for Date sorting as we want to search more recent messages first.
            var bSubjectSorted = (gDBView.sortType == nsMsgViewSortType.bySubject);
            var bWhoSorted = (gDBView.sortType == nsMsgViewSortType.byAuthor || gDBView.sortType == nsMsgViewSortType.byRecipient);
            var bAscendingSort = (gDBView.sortOrder == nsMsgViewSortOrder.ascending);
            var bDateSorted = (gDBView.sortType == nsMsgViewSortType.byDate);
            var row, inc;
            if (bAscendingSort && !bDateSorted)
            {
                row = 0;
                inc = 1;
            }
            else
            {
                row = numItems - 1;
                inc = -1;
            }
            // If doing a find again (user hit period key) then start at the item after the currently selected one 
            if (findAgain)
            {
                // Shift key held down means go the opposite direction
                if (event.shiftKey)
                    inc = -inc;
                row = selection.currentIndex + inc;
            }

            // Match the beginning of the text or white space, followed by zero or more punctuation characters, followed by the match string
            var matchExpression = new RegExp("(?:^|\\s)[\\x21-\\x2F\\x3A-\\x40\\x5B-\\x60\\x7B-\\x7E]*" + ttsBuffer, "i");
            var matchRow = -1;
            var whoCol = null;
            
            if (document.getElementById("whoCol").hidden === false)
                whoCol = treeBoxObj.columns.getNamedColumn("whoCol");
            
            for (; row >= 0 && row < numItems; row += inc)
            {
                // Don't bother searching Subject if its a secondary column
                // and we've already found a secondary match, or if
                // it isn't currently being shown
                if ((!bWhoSorted || matchRow < 0) && subjectCol)
                {
                    var subject = GetItemValue(view, row, subjectCol, "bySubject");
                    if (matchExpression.exec(subject))
                    {
                        if (!bWhoSorted)
                        {
                            matchRow = row;
                            break;
                        }
                        if (matchRow < 0)
                            matchRow = row;
                    }
                }
                
                // Don't bother searching Who if its a secondary column
                // and we've already found a secondary match, or if
                // it isn't currently being shown
                if ((bWhoSorted || matchRow < 0) && whoCol)
                {
                    var who = GetItemValue(view, row, whoCol, "byNone");
                    if (matchExpression.exec(who))
                    {
                        if (bWhoSorted)
                        {
                            matchRow = row;
                            break;
                        }
                        if (matchRow < 0)
                            matchRow = row;
                    }
                }
            }
            
            if (matchRow >= 0)
            {
                selection.select(matchRow);
                treeBoxObj.ensureRowIsVisible(matchRow);
            }
            else
            {
                var sound = Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound);
                sound.beep();
            }
        }
    }
}

function ThreadTreeOnKeyUp(event)
{
    // Under Windows and Linux the Alt key being released casuses the top level
    // menu to get the keyboard focus.  We need to eat that event if what was
    // just done was an alt+click group selection.
    if (eatNextAltUp && event.keyCode == event.DOM_VK_ALT)
    {
        eatNextAltUp = false;
        event.stopPropagation();
    }
}

var groupSelectIndexArray;

var groupSelectColumnHandler = 
{
    getCellText:         function(row, col) {},
    getSortStringForRow: function(hdr) {},
    isString:            function() { return false; },
    getCellProperties:   function(row, col, props) {},
    getImageSrc:         function(row, col) { return null; },
    getRowProperties:    function(row, properties) {},
    getSortLongForRow:   function(hdr) { return groupSelectIndexArray[hdr.messageKey]; }
};

var CreateDbObserver =
{
    observe: function(aMsgFolder, aTopic, aData)
    {
        // Add the hidden column for handling group-select sorting
        gDBView.addColumnHandler("groupSelectCol", groupSelectColumnHandler);
        
        // If we've never saved the column states for this mailbox, then set them to default values
        var folderDisplay = document.getElementById("tabmail").currentTabInfo.folderDisplay;
        var dbFolderInfo = aMsgFolder.msgDatabase.dBFolderInfo;
        const isNews = folderDisplay.view.isNewsFolder !== 0;
        const columnProperty = "penelopeColumnRemap";
        if (dbFolderInfo.getBooleanProperty(columnProperty, false) === false)
        {
            var states = {
                  threadCol:             { ordinal:  1, visible: isNews},
                  flaggedCol:            { ordinal:  3, visible: false },
                  receivedCol:           { ordinal:  5, visible: false },
                  accountCol:            { ordinal:  7, visible: false },
                  unreadCol:             { ordinal:  9, visible: false },
                  totalCol:              { ordinal: 11, visible: false },
                  statusCol:             { ordinal: 13, visible:  true },
                  unreadButtonColHeader: { ordinal: 15, visible: false },
                  priorityCol:           { ordinal: 17, visible:  true },
                  attachmentCol:         { ordinal: 19, visible:  true },
                  tagsCol:               { ordinal: 21, visible:  true },
                  senderCol:             { ordinal: 23, visible: false },
                  recipientCol:          { ordinal: 25, visible: false },
                  whoCol:                { ordinal: 27, visible:  true },
                  dateCol:               { ordinal: 29, visible:  true },
                  sizeCol:               { ordinal: 31, visible:  true },
                  junkStatusCol:         { ordinal: 33, visible:  true },
                  subjectCol:            { ordinal: 35, visible:  true },
                  idCol:                 { ordinal: 37, visible: false },
                  glodaWhyCol:           { ordinal: 39, visible: false },
                  glodaScoreCol:         { ordinal: 41, visible: false },
                  locationCol:           { ordinal: 43, visible: false }
            };
            // We need to both put the states in _savedColumnStates and
            // call setColumnStates() because the latter saves
            // asynchronously from what's in _savedColumnStates
            folderDisplay._savedColumnStates = states;
            folderDisplay.setColumnStates(states, true);
            dbFolderInfo.setBooleanProperty(columnProperty, true);
        }
    }
};

function GetCoreSubject(text)
{
    // Can't do this with a single reg ex as the "[Fwd: subject]" format
    // needs to strip off the trailing square bracket character

    var fwdRegEx = /\[fwd?:\s*(.*)\]/i;
    var initialBracketsRegEx = /^\[.*\]\s*(\S+.*)/i;    
    var prefixRegEx = /^\s*(?:(?:re|fwd?)[^a-z]*:\s*)+(.*)/i;
    while (true)
    {
        var match;
        while (match = fwdRegEx.exec(text))
            text = match[1];

        while (match = initialBracketsRegEx.exec(text))
            text = match[1];

        if (match = prefixRegEx.exec(text))
            text = match[1];
        else
            break;
    }
    
    return text;
}

function GetItemValue(view, row, col, sortName)
{
    var msgKey;
    var hdr;
    switch (sortName)
    {
    case "bySubject":
        return GetCoreSubject(view.getCellText(row, col));
        
    case "byThread":
        return view.isContainer(row);
        
    case "byPriority":
        msgKey = gDBView.getKeyAt(row);
        hdr = gDBView.db. GetMsgHdrForKey(msgKey);
        return hdr.priority;
        
    case "byFlagged":
        msgKey = gDBView.getKeyAt(row);
        hdr = gDBView.db. GetMsgHdrForKey(msgKey);
        return hdr.isFlagged;
        
    case "byJunkStatus":
        msgKey = gDBView.getKeyAt(row);
        hdr = gDBView.db. GetMsgHdrForKey(msgKey);
        var junkScoreStr = hdr.getStringProperty("junkscore");
        var junkThreshold = pref.getIntPref("mail.adaptivefilters.junk_threshold");
        return parseInt(junkScoreStr, 10) >= junkThreshold;
        
    case "byAttachments":
        msgKey = gDBView.getKeyAt(row);
        hdr = gDBView.db. GetMsgHdrForKey(msgKey);
        return (hdr.flags & 0x10000000);  // 0x10000000 is MSG_FLAG_ATTACHMENT
        
    default:
        return view.getCellText(row, col);
    }
}

function ConvertColumnIDToSortName(columnID)
{
  const columnMap = {dateCol:               'byDate',
                     receivedCol:           'byReceived',
                     senderCol:             'byAuthor',
                     recipientCol:          'byRecipient',
                     whoCol:                'byWho',
                     subjectCol:            'bySubject',
                     locationCol:           'byLocation',
                     accountCol:            'byAccount',
                     unreadButtonColHeader: 'byUnread',
                     statusCol:             'byStatus',
                     sizeCol:               'bySize',
                     priorityCol:           'byPriority',
                     flaggedCol:            'byFlagged',
                     threadCol:             'byThread',
                     tagsCol:               'byTags',
                     junkStatusCol:         'byJunkStatus',
                     idCol:                 'byId',
                     attachmentCol:         'byAttachments'};

  var sortName;
  if (columnID in columnMap)
  {
      sortName = columnMap[columnID];
  }
  else
  {
      // If the column isn't in the map, check and see if it's a custom column
      try
      {
          // try to grab the columnHandler (an error is thrown if it does not exist)
          columnHandler = gDBView.getColumnHandler(columnID);

          // it exists - save this column ID in the customSortCol property of
          // dbFolderInfo for later use (see nsIMsgDBView.cpp)
          gDBView.db.dBFolderInfo.setProperty('customSortCol', columnID);

          sortName = "byCustom";
      }
      catch(err)
      {
          dump("unsupported sort column: " + columnID + " - no custom handler installed. (Error was: " + err + ")\n");
      }
  }

  return sortName;
}

function MailboxGroupSelect(tree, row, col, moveItems)
{
    var view = tree.view;
    var treeBoxObj = tree.treeBoxObject;
    var selection = view.selection;
    var sortName = ConvertColumnIDToSortName(col.id);
    var compareValue = GetItemValue(view, row, col, sortName);
    var numItems = view.rowCount;

    selection.selectEventsSuppressed = true;
    selection.clearSelection();

    var MatchBeforeNum = 0;
    var MatchAfterNum = 0;
    var r;
    for (r = 0; r < numItems; r++)
    {
        if (GetItemValue(view, r, col, sortName) == compareValue)
        {
            // Need to use rangedSelect() as it doesn't affect existing selection
            selection.rangedSelect(r, r, true);

            if (r < row)
                MatchBeforeNum++;
            else if (r > row)
                MatchAfterNum++;
        }
    }

    if (moveItems && (MatchBeforeNum || MatchAfterNum))
    {
        var NonMatchBeforeIndex = 0;
        var MatchBeforeIndex =    row - MatchBeforeNum;
        var NonMatchAfterIndex =  row + MatchAfterNum + 1;
        var MatchAfterIndex =     row + 1;

        groupSelectIndexArray = new Array();

        for (r = 0; r < numItems; r++)
        {
            var newIndex = 0;
            if (GetItemValue(view, r, col, sortName) == compareValue)
            {
                if (r < row)
                    newIndex = MatchBeforeIndex++;
                else if (r > row)
                    newIndex = MatchAfterIndex++;
                else
                    newIndex = row;
            }
            else
            {
                if (r < row)
                    newIndex = NonMatchBeforeIndex++;
                else
                    newIndex = NonMatchAfterIndex++;
            }
            var msgKey = gDBView.getKeyAt(r);
            groupSelectIndexArray[msgKey] = newIndex;
        }

        sortName = ConvertColumnIDToSortName("groupSelectCol");

        // Force sort to occur by thinking that the view is currently not sorted
        gDBView.sortType = nsMsgViewSortType.byNone;
        MsgSortThreadPane(sortName);
        // Consider not sorted when done
        gDBView.sortType = nsMsgViewSortType.byNone;

        groupSelectIndexArray = null;

        // Make sure item clicked on is visible
        treeBoxObj.ensureRowIsVisible(row);
    }

    selection.currentIndex = row;
    selection.selectEventsSuppressed = false;
}


function setNewMailSound()
{
    if (gPrefBranch.getBoolPref("mail.biff.play_sound") && !gPrefBranch.prefHasUserValue("mail.biff.play_sound.url"))
    {
        // Get the install location of the extension
        const id= "{D1D37B8A-4F3C-11DB-8373-B622A1EF5492}";
        var extDir = Components.classes["@mozilla.org/extensions/manager;1"]
                        .getService(Components.interfaces.nsIExtensionManager)
                        .getInstallLocation(id)
                        .getItemLocation(id);

        // Set the preferences for the sound to play for new mail
        var newmail = "file://"+extDir.path+"/newmail.wav";
        gPrefBranch.setIntPref("mail.biff.play_sound.type", 1);
        gPrefBranch.setCharPref("mail.biff.play_sound.url", newmail);
    }
}


function penelopeShowMailbox()
{
    var selectedFolders = gFolderTreeView.getSelectedFolders();
    RestoreSelectionWithoutContentLoad(gFolderTree);
    gFolderTreeView.selectFolder(selectedFolders[0]);
    gOpeningMailboxTab = true;
    penelopeFolderPaneSelectionChange();
    gOpeningMailboxTab = false;
}

var gFirstSelectionChange = true;
var gMailboxesTitle = null;

function penelopeFolderPaneSelectionChange()
{
    if (gIsMailboxesWindow)
    {
        if (!gMailboxesTitle)
        {
            var strBundle = document.getElementById("penelopeStrings");
            if (strBundle)
                gMailboxesTitle = strBundle.getString("mailboxesTitle");
        }
        document.title = gMailboxesTitle;
    }
    else
    {
        if (!gOpeningMailboxTab && !gFirstSelectionChange && !gPrefBranch.getBoolPref("penelope.mailbox.selectShowsMailbox"))
            gIgnoreSyntheticFolderPaneSelectionChange = true;
        FolderPaneSelectionChange();

        var tabmail = document.getElementById('tabmail');
        var tabInfo = tabmail.currentTabInfo;

        if (!("needsFUMLUBSelection" in tabInfo) || tabInfo.needsFUMLUBSelection)
        {
            var folder = tabInfo.folderDisplay.displayedFolder;
            if (folder)
            {
                SelectFUMLUB(folder);
                tabInfo.needsFUMLUBSelection = false;
            }
        }
    }
    gFirstSelectionChange = false;
}
