/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the Mozilla Penelope project.
 *
 * The Initial Developer of the Original Code is
 * QUALCOMM Incorporated.
 * Portions created by QUALCOMM Incorporated are
 * Copyright (C) 2009-2010 QUALCOMM Incorporated. All Rights Reserved.
 *
 * Contributor(s):
 *     Jeff Beckley <beckley@qualcomm.com> original author
 *     Dale Wiggins <dwiggins@qualcomm.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

Components.utils.import("resource://app/modules/searchSpec.js");

function penelopeSearchOnLoad()
{
  searchOnLoad();
  
  updateMailboxCss();
  enableMailboxLines();
  updateIconColumns();

  pref = Components.classes["@mozilla.org/preferences-service;1"]
      .getService(Components.interfaces.nsIPrefBranch);
  var indexerEnabled = pref.getBoolPref("mailnews.database.global.indexer.enabled");
  // Only allow the indexed search checkbox to be enabled if Gloda pref is on.
  var checkUseIndexedSearch = document.getElementById("checkUseIndexedSearch");
  if (checkUseIndexedSearch)
  {
    checkUseIndexedSearch.disabled = !indexerEnabled;
    try
    {
      checkUseIndexedSearch.checked =
          (indexerEnabled && pref.getBoolPref("mailnews.search_dialog.use_indexed_search"));
    } catch(e) {}
  }

  // Get rid of the "Search for messages in:" label and replace it with the
  // "Limit search to" checkbox.  We have to do this in code rather than in
  // the XUL because the TB core XUL doesn't have IDs on the elements we
  // want to modify.
  var searchableFolders = document.getElementById("searchableFolders");
  var parent = searchableFolders.parentNode;
  parent.removeChild(parent.firstChild);
  
  var limitSearchTo = document.getElementById("limitSearchTo");
  limitSearchTo.parentNode.removeChild(limitSearchTo);
  parent.insertBefore(limitSearchTo, parent.firstChild);

  // Hide the "Run search on server" if no servers to run on!
  var checkSearchOnline = document.getElementById("checkSearchOnline");
  if (checkSearchOnline)
  {
    var accountManager = Components.classes["@mozilla.org/messenger/account-manager;1"]
                         .getService(Components.interfaces.nsIMsgAccountManager);
    var foundSearchServer = false;
    for each (var server in fixIterator(accountManager.allServers, Components.interfaces.nsIMsgIncomingServer))
    {
      if (server.offlineSupportLevel)
      {
        foundSearchServer = true;
        break;
      }
    }
    if (!foundSearchServer)
      checkSearchOnline.hidden = true;
  }
  
  setupIndexedSearchItems(true);
}

function penelopeOpenInFolderButton()
{
  // Get the closet mail window and open up the mailbox and select the message
  // from it. It will open in a tab or a window based on the setting.
  let mailWindow = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService()
                             .QueryInterface(Components.interfaces.nsIWindowMediator)
                             .getMostRecentWindow("mail:3pane");

  if (mailWindow)
  {
    let msgHdr = gFolderDisplay.selectedMessage;
    let folderUri = msgHdr.folder.URI;

    mailWindow.goMailbox(folderUri, false, false, false, false, msgHdr);
  }
}

window.origOnSearch = window.onSearch;
window.onSearch = penOnSearch;

function penOnSearch()
{
  var useIndexedSearch = document.getElementById("checkUseIndexedSearch");
  gFolderDisplay.view.search.useGloda = useIndexedSearch ? useIndexedSearch.checked : false;

  var limitSearchTo = document.getElementById("limitSearchTo");
  gFolderDisplay.view.search.searchEverywhere = limitSearchTo ? !limitSearchTo.checked : false;
  
  var searchSubFolders = document.getElementById("checkSearchSubFolders");
  gFolderDisplay.view.search.searchSubFolders = searchSubFolders ? searchSubFolders.checked : false;

  origOnSearch();
}

function setupIndexedSearchItems(isIndexChange)
{
  var useIndexedSearch = document.getElementById("checkUseIndexedSearch");
  var useIndexedSearchChecked = useIndexedSearch ? useIndexedSearch.checked : false;
  var checkSearchOnline = document.getElementById("checkSearchOnline");
  var searchSubFolders = document.getElementById("checkSearchSubFolders");
  var searchFoldersLabel = document.getElementById("searchFoldersLabel");
  var limitSearchTo = document.getElementById("limitSearchTo");
  var searchableFolders = document.getElementById("searchableFolders");

  origUpdateSearchFolderPicker(gCurrentFolder.URI);

  if (useIndexedSearchChecked)
  {
    // Force the scope to be Gloda to cause correct search terms to appear.
    forceGlodaScope();
    setSearchScope(Components.interfaces.nsMsgSearchScope.offlineGlodaMail);

    // Disable the "Run search on server" option.
    if (checkSearchOnline)
    {
      checkSearchOnline.disabled = true;
      checkSearchOnline.checked = false;
    }

    if (limitSearchTo && searchableFolders)
    {
      limitSearchTo.disabled = false;
      searchableFolders.disabled = !limitSearchTo.checked;
      searchSubFolders.disabled = !limitSearchTo.checked;
    }
  }
  else
  {
    // Force the scope to be not Gloda to cause correct search terms to appear.
    unforceGlodaScope();
    setSearchScope(GetScopeForFolder(gCurrentFolder));

    if (limitSearchTo)
    {
      if (isIndexChange)
        limitSearchTo.checked = true;
      limitSearchTo.disabled = true;
    }
    if (searchableFolders)
      searchableFolders.disabled = false;
  }
}

window.origUpdateSearchFolderPicker = window.updateSearchFolderPicker;
window.updateSearchFolderPicker = penUpdateSearchFolderPicker;

function penUpdateSearchFolderPicker(folderURI)
{
  origUpdateSearchFolderPicker(folderURI);

  setupIndexedSearchItems(false);
}
